<?php
/**
 * @package     JDIdeal
 * @subpackage  Addon
 *
 * @author      Roland Dalmulder <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2021 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace Jdideal\Addons;

defined('_JEXEC') or die;

use JDatabaseDriver;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use RuntimeException;
use stdClass;

/**
 * RSDirectory! addon.
 *
 * @package     JDIdeal
 * @subpackage  RsDirectory
 * @since       2.13
 */
class AddonRseventspro implements AddonInterface
{
	/**
	 * Database interface
	 *
	 * @var    JDatabaseDriver
	 * @since  5.3.0
	 */
	private $db;

	/**
	 * Constructor.
	 *
	 * @since  5.3.0
	 */
	public function __construct()
	{
		$this->db = Factory::getDbo();
	}

	/**
	 * Get the name of the addon.
	 *
	 * @return  string  The name of the component.
	 *
	 * @since   5.3.0
	 */
	public function getName(): string
	{
		return 'RSEvents! Pro';
	}

	/**
	 * Collect the order information.
	 *
	 * @param   string  $orderId  The ID to get the order data for
	 * @param   array   $data     Array with payment info
	 *
	 * @return  array  Array with order information.
	 *
	 * @since   2.13.0
	 *
	 * @throws  RuntimeException
	 */
	public function getOrderInformation(string $orderId, array $data): array
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('email') . ',' . $this->db->quoteName('state'))
			->from($this->db->quoteName('#__rseventspro_users'))
			->where($this->db->quoteName('id') . ' = ' . $orderId);
		$this->db->setQuery($query);
		$order = $this->db->loadObject();

		if (!$order)
		{
			$data['order_status'] = false;

			return $data;
		}

		$data['order_status'] = $this->translateOrderStatus($order->state);
		$data['user_email']   = $order->email;

		// Return the data
		return $data;
	}

	/**
	 * Translate component order status to RO Payments status.
	 *
	 * @param   string  $orderStatus  The component order status.
	 *
	 * @return  string  The RO Payments order status.
	 *
	 * @since   2.13.0
	 */
	public function translateOrderStatus(string $orderStatus): string
	{
		switch ($orderStatus)
		{
			case '0':
				return 'P';
				break;
			case '1':
				return 'C';
				break;
			case '2':
				return 'X';
				break;
			default:
				return 'P';
				break;
		}
	}

	/**
	 * Collect customer information.
	 *
	 * @param   string  $orderId  The ID of the order to get the info from.
	 *
	 * @return  array  The array of customer information.
	 *
	 * @since   2.13.0
	 */
	public function getCustomerInformation(string $orderId): array
	{
		return [];
	}

	/**
	 * Get the component order status name.
	 *
	 * @param   string  $data  The name of the RO Payments order status.
	 *
	 * @return  string  The name of the component order status.
	 *
	 * @since   2.13.0
	 */
	public function getOrderStatusName(array $data): string
	{
		switch ($data['order_status'])
		{
			case 'C':
				return Text::_('COM_ROPAYMENTS_STATUS_CONFIRMED');
				break;
			case 'P':
				return Text::_('COM_ROPAYMENTS_STATUS_PENDING');
				break;
			case 'X':
				return Text::_('COM_ROPAYMENTS_STATUS_CANCELLED');
				break;
			default:
				return Text::_('COM_ROPAYMENTS_STATUS_PENDING');
				break;
		}
	}

	/**
	 * Get the link to the component.
	 *
	 * @return  string  The link to the component.
	 *
	 * @since   2.13.0
	 */
	public function getComponentLink(): string
	{
		return 'index.php?option=com_rseventspro';
	}

	/**
	 * Get the link to the order on the administrator side.
	 *
	 * @param   string  $orderId  The order ID to get the link for.
	 *
	 * @return  string  The link to the order.
	 *
	 * @since   2.13.0
	 */
	public function getAdminOrderLink(string $orderId): string
	{
		return 'index.php?option=com_rseventspro&task=subscription.edit&id=' . $orderId;
	}

	/**
	 * Get the link to the order on the frontend.
	 *
	 * @param   string  $orderId      The order ID for the link
	 * @param   string  $orderNumber  The order number for the link
	 *
	 * @return  string  The link to the order.
	 *
	 * @since   2.13.0
	 */
	public function getOrderLink(string $orderId, string $orderNumber): string
	{
		return 'index.php?option=com_rseventspro&view=rseventspro&layout=subscriptions';
	}

	/**
	 * Perform the callback for any additional tasks to be done.
	 *
	 * @param   array  $data  An array of order data.
	 *
	 * @return  array  An array of order data.
	 *
	 * @since   2.13.0
	 */
	public function callBack(array $data): array
	{
		return [];
	}

	/**
	 * Replace addon specific placeholders.
	 *
	 * @param   stdClass  $details  The transaction details
	 * @param   string    $message  The message to replace the placeholders for
	 *
	 * @return  string  The message with replaced placeholders.
	 *
	 * @since   5.3.0
	 */
	public function replacePlaceholders(stdClass $details, string $message): string
	{
		$find   = [];
		$find[] = '{eventName}';
		$db     = $this->db;
		$query  = $db->getQuery(true)
			->select(
				$db->quoteName(
					[
						'events.name'
					],
					[
						'eventName'
					]
				)
			)
			->from($db->quoteName('#__rseventspro_users', 'users'))
			->leftJoin(
				$db->quoteName('#__rseventspro_events', 'events')
				. ' ON ' . $db->quoteName('events.id') . ' = ' . $db->quoteName('users.ide')
			)
			->where($db->quoteName('users.id') . ' = ' . (int) $details->order_id);
		$db->setQuery($query);
		$replace = $db->loadColumn();

		return str_ireplace($find, $replace, $message);
	}
}
