<?php
/**
 * @package    JDiDEAL
 *
 * @author     Roland Dalmulder <contact@rolandd.com>
 * @copyright  Copyright (C) 2009 - 2021 RolandD Cyber Produksi. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link       https://rolandd.com
 */

defined('_JEXEC') or die;

use Jdideal\Gateway;
use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\FileLayout;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;

/**
 * RSEventsPro RO Payments plugin.
 *
 * @package  JDiDEAL
 * @since    2.8.0
 */
class PlgSystemRseproJdideal extends CMSPlugin
{
	/**
	 * Set the value of the payment option
	 *
	 * @var    string
	 * @since  2.8
	 */
	public $rsprooption = 'jdideal';

	/**
	 * Auto-load the language
	 *
	 * @var    string
	 * @since  4.2.0
	 */
	protected $autoloadLanguage = true;

	/**
	 * Load the site application
	 *
	 * @var    JApplicationSite
	 * @since  4.5.0
	 */
	protected $app;

	/**
	 * Load the database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  4.5.0
	 */
	protected $db;

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe
	 * @param   array   $config    An optional associative array of configuration settings.
	 *                             Recognized key values include 'name', 'group', 'params', 'language'
	 *                             (this list is not meant to be comprehensive).
	 *
	 * @since   2.8
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$this->_plugin = PluginHelper::getPlugin('system', 'rseprojdideal');
		$this->_params = new Registry($this->_plugin->params);
	}

	/*
	*	Is RSEvents!Pro installed
	*/
	/**
	 * Check if RSEvents!Pro installed.
	 *
	 * @return  boolean  True if installed | False if not installed.
	 *
	 * @since   2.8
	 */
	public function canRun(): bool
	{
		return file_exists(JPATH_SITE . '/components/com_rseventspro/helpers/rseventspro.php');
	}

	/*
	*	Add the current payment option to the Payments List
	*/

	/**
	 * Add the current payment option to the Payments List.
	 *
	 * @return  string  HTML select option.
	 *
	 * @since   2.8.0
	 *
	 * @throws  InvalidArgumentException
	 */
	public function rsepro_addOptions()
	{
		$lang = Factory::getLanguage();
		$lang->load('plg_system_rseprojdideal', JPATH_ADMINISTRATOR);

		if ($this->canRun())
		{
			return HTMLHelper::_('select.option', $this->rsprooption, Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_NAME'));
		}

		return HTMLHelper::_('select.option', '', '');
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @param   array  $vars  An array with payment details.
	 *
	 * @return  string  HTML select option.
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_addOptions($vars)
	{
		return $this->rsepro_addOptions($vars);
	}

	/**
	 * Add optional fields for the payment plugin. Example: Credit Card Number, etc.
	 *    Please use the syntax <form method="post" action="index.php?option=com_rseventspro&task=process" name="paymentForm">
	 *    The action provided in the form will actually run the rsepro_processForm() of your payment plugin.
	 *
	 * @param   array  $vars  An array with payment options.
	 *
	 * @return  mixed  Null if this plugin is not used | String with form options.
	 *
	 * @since   2.8.0
	 *
	 * @throws  Exception
	 */
	public function rsepro_showForm($vars)
	{
		$app = Factory::getApplication();

		if ($app->getName() !== 'site')
		{
			return;
		}

		// Check to see if we can show something
		if (!$this->canRun())
		{
			return;
		}

		// Empty the Joomla message queue
		Factory::getApplication()->getMessageQueue(true);

		if (array_key_exists('method', $vars) && $vars['method'] === $this->rsprooption)
		{
			$lang = Factory::getLanguage();
			$lang->load('plg_system_rseprojdideal', JPATH_ADMINISTRATOR);
			$lang->load('com_rseventspro', JPATH_SITE);

			// Is the plugin enabled ?
			$enable = PluginHelper::isEnabled('system', 'rseprojdideal');

			if (!$enable)
			{
				return;
			}

			$details = $vars['details'];
			$tickets = $vars['tickets'];

			// Check to see if its a payment request
			if (empty($details->verification) && empty($details->ide) && empty($details->email) && empty($tickets))
			{
				return;
			}

			require_once JPATH_SITE . '/components/com_rseventspro/helpers/rseventspro.php';

			$html = '';
			$html .= '<h2 class="componentheading">' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_NAME') . '</h2>' . "\n";
			$html .= '<div class="rs_payment_jdideal">' . "\n";
			$html .= '<fieldset>' . "\n";
			$html .= '<table cellspacing="10" cellpadding="0">' . "\n";
			$html .= '<tr>' . "\n";
			$html .= '<td>' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_TICKETS') . '</td>' . "\n";
			$html .= '<td>' . "\n";

			$discount = $details->discount;
			$total = 0;

			if (!empty($tickets))
			{
				foreach ($tickets as $ticket)
				{
					if (empty($ticket->price))
					{
						$html .= $ticket->quantity . ' x ' . $ticket->name . ' (' . Text::_('RSEPRO_FREE') . ')<br />';
					}
					else
					{
						$html .= $ticket->quantity . ' x ' . $ticket->name . ' (' . rseventsproHelper::currency($ticket->price) . ')<br />';
					}

					if ($ticket->price > 0)
					{
						$total += ($ticket->quantity * $ticket->price);
					}
				}
			}

			if (!empty($discount))
			{
				$total -= $discount;
			}

			if (!empty($discount))
			{
				$html .= '<tr>' . "\n";
				$html .= '<td>' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_TICKETS_DISCOUNT') . '</td>' . "\n";
				$html .= '<td>' . rseventsproHelper::currency($discount) . '</td>' . "\n";
				$html .= '</tr>' . "\n";
			}

			if ($details->early_fee)
			{
				$total = $total - $details->early_fee;
				$html .= '<tr>' . "\n";
				$html .= '<td>' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_EARLY_FEE') . '</td>' . "\n";
				$html .= '<td>' . "\n";
				$html .= rseventsproHelper::currency($details->early_fee);
				$html .= '</td>' . "\n";
				$html .= '</tr>' . "\n";
			}

			if ($details->late_fee)
			{
				$total = $total + $details->late_fee;
				$html .= '<tr>' . "\n";
				$html .= '<td>' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_LATE_FEE') . '</td>' . "\n";
				$html .= '<td>' . "\n";
				$html .= rseventsproHelper::currency($details->late_fee);
				$html .= '</td>' . "\n";
				$html .= '</tr>' . "\n";
			}

			if (!empty($details->tax))
			{
				$total += $details->tax;
				$html .= '<tr>' . "\n";
				$html .= '<td>' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_TICKETS_TAX') . '</td>' . "\n";
				$html .= '<td>' . rseventsproHelper::currency($details->tax) . '</td>' . "\n";
				$html .= '</tr>' . "\n";
			}

			$html .= '<tr>' . "\n";
			$html .= '<td>' . Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_TICKETS_TOTAL') . '</td>' . "\n";
			$html .= '<td>' . rseventsproHelper::currency($total) . '</td>' . "\n";
			$html .= '</tr>' . "\n";

			$html .= '</table>' . "\n";
			$html .= '</fieldset>' . "\n";

			// Load the parameters
			$parameters = $this->loadParameters($details->ide);
			$alias      = $this->params->get('profileAlias');

			if (is_array($parameters) && array_key_exists('profileAlias', $parameters))
			{
				$alias = $parameters['profileAlias'];
			}

			// Set some needed data
			$data = [
					'amount'		=> $total,
					'order_number'	=> $details->id,
					'order_id'		=> $details->id,
					'origin'		=> 'rseventspro',
					'return_url'	=> rseventsproHelper::route(Uri::root() . 'index.php?option=com_rseventspro&task=process&customer=1', false),
					'notify_url'	=> rseventsproHelper::route(Uri::root() . 'index.php?option=com_rseventspro&task=process', false),
					'cancel_url'	=> '',
					'email'	        => '',
					'payment_method' => '',
					'profileAlias'  => $alias,
					'custom_html'	=> $html,
					'silent'        => false,
			];

			// Build the form
			ob_start();

			// Include the payment form
			$layout = new FileLayout('forms.form', null, ['component' => 'com_jdidealgateway']);
			echo $layout->render(['data' => $data]);
			$html = ob_get_contents();

			ob_end_clean();

			echo $html;
		}
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @param   array  $vars  An array with payment details.
	 *
	 * @return  mixed  Null if this plugin is not used | String with form options.
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_showForm($vars)
	{
		return $this->rsepro_showForm($vars);
	}

	/**
	 * Process the form and check the payment result.
	 *
	 * @param   array  $vars  An array with payment details.
	 *
	 * @return  mixed  Null when this plugin is not used | Redirect with payment status.
	 *
	 * @since   2.8
	 *
	 * @throws  Exception
	 * @throws  RuntimeException
	 * @throws  InvalidArgumentException
	 */
	public function rsepro_processForm($vars)
	{
		// Check to see if we need to handle this request.
		if (!$this->canRun())
		{
			return;
		}

		require_once JPATH_SITE . '/components/com_rseventspro/helpers/rseventspro.php';

		$db = Factory::getDbo();
		$app  = Factory::getApplication();

		JLoader::registerNamespace('Jdideal', JPATH_LIBRARIES);

		if (class_exists(Gateway::class) === false)
		{
			JLoader::registerNamespace('Jdideal', JPATH_LIBRARIES . '/Jdideal');
		}

		$jinput = Factory::getApplication()->input;
		$jdideal = new Gateway;

		$trans = $jinput->get('transactionId');
		$column = 'trans';

		if (empty($trans))
		{
			$trans  = $jinput->get('pid');
			$column = 'pid';
		}

		$details = $jdideal->getDetails($trans, $column, false, 'rseventspro');
		$isValid = $jdideal->isValid($details->result);

		// Set the status
		switch ($jdideal->getStatusCode($details->result))
		{
			case 'C':
				$newStatus = '1';
				break;
			case 'X':
				$newStatus = '2';
				break;
			default:
				$newStatus = '0';
				break;
		}

		// Get the event
		$query = $db->getQuery(true)
			->select($db->quoteName('rseventspro_events.id'))
			->select($db->quoteName('rseventspro_events.name'))
			->from($db->quoteName('#__rseventspro_events', 'rseventspro_events'))
			->leftJoin(
				$db->quoteName('#__rseventspro_users', 'rseventspro_users')
				. ' ON ' . $db->quoteName('rseventspro_users.ide') . ' = ' . $db->quoteName('rseventspro_events.id')
			)
			->where($db->quoteName('rseventspro_users.id') . ' = ' . (int) $details->order_id);
		$db->setQuery($query);
		$event = $db->loadObject();

		// Build the URLs
        $url = rseventsproHelper::route('index.php?option=com_rseventspro&layout=default', false);

        // Check if we have event details
        if ($event)
        {
            $url = rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id=' . rseventsproHelper::sef($event->id, $event->name), false);
        }

		// Load the customer made message
		$message = $jdideal->getMessage($details->id);

		// Check if we are the customer
		if ($app->input->getInt('customer', 0) === 1)
		{
			if ((int) $newStatus === 1)
			{
				if (!$message)
				{
					$message = Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_PAYMENT_RECEIVED');
				}

				// Check if we have event details
                $app->redirect($url, $message);
			}

			if (!$message)
			{
				$message = Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_INVALID_SUBSCRIPTION');
			}

			$app->redirect($url, $message, 'error');
		}

		$jdideal->log('Update ticket', $details->id);

		// Update the ticket
		$query->clear()
			->update($db->quoteName('#__rseventspro_users'))
			->set($db->quoteName('state') . ' = ' . $db->quote($newStatus))
			->set($db->quoteName('ideal') . ' = ' . $db->quote($details->card))
			->where($db->quoteName('id') . ' = ' . (int) $details->order_id);
		$db->setQuery($query);

		if ($db->execute())
		{
			// Check if the payment is OK
			if ($isValid)
			{
				// Send the activation email and confirm the order status
				require_once JPATH_SITE . '/components/com_rseventspro/helpers/emails.php';

				rseventsproHelper::confirm($details->order_id);

				if (!$message)
				{
					$message = Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_PAYMENT_RECEIVED');
				}

				$app->redirect($url, $message);
			}
			else
			{
				if (!$message)
				{
					$message = Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_SUBSCRIPTION_PAYMENT_FAILED');
				}

                $app->redirect($url, $message, 'error');
			}
		}
		else
		{
			if (!$message)
			{
				$message = Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_INVALID_SUBSCRIPTION');
			}

            $app->redirect($url, $message, 'error');
		}
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @param   array  $vars  An array with payment details.
	 *
	 * @return  mixed  Null when this plugin is not used | Redirect with payment status.
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_processForm($vars)
	{
		return $this->rsepro_processForm($vars);
	}

	/**
	 * Handle the tax settings.
	 *
	 * @param   array  $vars  The tax settings.
	 *
	 * @return  mixed  Null if this plugin is not used | Float if used.
	 *
	 * @since   2.8
	 */
	public function rsepro_tax($vars)
	{
		if (array_key_exists('method', $vars) && $vars['method'] === $this->rsprooption)
		{
			require_once JPATH_SITE . '/components/com_rseventspro/helpers/rseventspro.php';

			$total = array_key_exists('total', $vars) ? $vars['total'] : 0;
			$tax_value = $this->_params->get('tax_value', 0);
			$tax_type = $this->_params->get('tax_type', 0);

			return rseventsproHelper::setTax($total, $tax_type, $tax_value);
		}

		return null;
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @param   array  $vars  The tax settings.
	 *
	 * @return  mixed  Null if this plugin is not used | Float if used.
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_tax($vars)
	{
		return $this->rsepro_tax($vars);
	}

	/**
	 * Store the RO Payments parameters when the event is being saved.
	 *
	 * @param   array  $data  The event data
	 *
	 * @return  void
	 *
	 * @since   4.2.0
	 */
	public function rsepro_afterEventStore($data): void
	{
		$settings = $this->app->input->get('jdidealParams', [], 'array');
		$eventId  = $data['event']->getEvent()->id;

		// Store the settings for the event
		$tables = $this->db->getTableList();
		$table = $this->db->getPrefix() . 'rseventspro_jdideal';

		if (!in_array($table, $tables, true))
		{
			return;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('params'))
			->from($this->db->quoteName('#__rseventspro_jdideal'))
			->where($this->db->quoteName('event_id') . ' = ' . (int) $eventId);
		$this->db->setQuery($query);
		$params = $this->db->loadResult();

		if ($params)
		{
			$query->clear()
				->update($this->db->quoteName('#__rseventspro_jdideal'))
				->set($this->db->quoteName('params') . ' = ' . $this->db->quote(json_encode($settings)))
				->where($this->db->quoteName('event_id') . ' = ' . (int) $eventId);
		}
		else
		{
			$query->clear()
				->insert($this->db->quoteName('#__rseventspro_jdideal'))
				->columns(['event_id', 'params'])
				->values($eventId . ',' . $this->db->quote(json_encode($settings)));
		}

		$this->db->setQuery($query);

		try
		{
			$this->db->execute();
		}
		catch (Exception $e)
		{
			$this->app->enqueueMessage($e->getMessage(), 'error');
		}
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @param   array  $data  The event data
	 *
	 * @return  void
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_afterEventStore($data): void
	{
		$this->rsepro_afterEventStore($data);
	}

	/**
	 * Add a menu option to the edit event page.
	 *
	 * @return  void
	 *
	 * @since   4.2.0
	 */
	public function rsepro_addMenuOption(): void
	{
		$path = __DIR__ . '/configuration.xml';

		// Check if params file exists
		if (!file_exists($path))
		{
			return;
		}

		// Load the RO Payments stylesheet
		HTMLHelper::stylesheet('com_jdidealgateway/jdidealgateway.css', ['relative' => true]);

		// Load the data
		$data = ['jdidealParams' => $this->loadParameters($this->app->input->getInt('id'))];

		// Load the specific plugin parameters
		$form = new Form('rsfpjdideal');
		$form->loadFile(__DIR__ . '/configuration.xml');
		$form->bind($data);
		$formLayout = preg_replace("/[\r|\n]/", '', $form->renderFieldset('formConfig'));
		$document   = Factory::getDocument();
		$document->addScriptDeclaration(
				<<<JS
jQuery(document).ready(function(){
				// Check if we are already active
				var active = '';
				if (jQuery('ul#rsepro-edit-menu > li.active').children().data('target') === '#rsepro-edit-tabjdideal') {
					active = 'active';
				}
				jQuery('form > div.tab-content:first').append('<div class="tab-pane ' + active + '" id="rsepro-edit-tabjdideal" style="">{$formLayout}</div>');
			});

			function showForm(element) {
				// Change class on navigation
				jQuery('ul#rsepro-edit-menu > li.active').removeClass('active');
				jQuery(element).addClass('active');

				// Change class on content
				jQuery('div.tab-content > div.active').removeClass('active');
				jQuery(jQuery(element).data('target')).addClass('active');
			}
JS
);
		echo '<li class="respro-hide" style="display: block"><a href="javascript:void(0);" onclick="showForm(this);" '
			. 'data-target="#rsepro-edit-tabjdideal" data-toggle="tab">' . Text::_('PLG_SYSTEM_RSEPRO_TAB_JDIDEAL')
			. ' <span class="jdicon-jdideal"></span></a></li>';
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @return  void
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_addMenuOption($vars)
	{
		$this->rsepro_addMenuOption($vars);
	}

	/**
	 * Load the settings for a given event.
	 *
	 * @param   int  $eventId  The ID of the event to get the parameters for.
	 *
	 * @return  array  Array of settings.
	 *
	 * @since   4.2.0
	 */
	private function loadParameters($eventId): ?array
	{
		if (!$eventId)
		{
			return [];
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('params'))
			->from($this->db->quoteName('#__rseventspro_jdideal'))
			->where($this->db->quoteName('event_id') . ' = ' . (int) $eventId);
		$this->db->setQuery($query);

		return json_decode($this->db->loadResult(), true);
	}

	/**
	 * Return the name of the selected payment.
	 *
	 * @param   array  $vars  Contains the gateway name
	 *
	 * @return  string|void  The name of the selected payment | Nothing if we are not to handle the request.
	 *
	 * @since   4.13.1
	 */
	public function rsepro_name($vars)
	{
		if (!$this->canRun())
		{
			return;
		}

		if (isset($vars['gateway']) && $vars['gateway'] === $this->rsprooption)
		{
			return Text::_('RSEPRO_PLG_PLUGIN_JDIDEAL_NAME');
		}
	}

	/**
	 * Proxy function for RSEvents since version 1.13.3.
	 *
	 * @return  string|void  The name of the selected payment | Nothing if we are not to handle the request.
	 *
	 * @since   6.2.1
	 */
	public function onrsepro_name($vars)
	{
		return $this->rsepro_name($vars);
	}
}
